@@ -20,7 +20,6 @@ import getpass
 import io
 import logging
 import os
-import sys
 
 from novaclient.v2 import servers
 from osc_lib.cli import parseractions
@@ -32,6 +31,7 @@ import six
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import common as network_common
 
 
 LOG = logging.getLogger(__name__)
@@ -189,12 +189,6 @@ def _prep_server_detail(compute_client, image_client, server):
     return info
 
 
-def _show_progress(progress):
-    if progress:
-        sys.stdout.write('\rProgress: %s' % progress)
-        sys.stdout.flush()
-
-
 class AddFixedIP(command.Command):
     _description = _("Add fixed IP address to server")
 
@@ -234,11 +228,10 @@ class AddFixedIP(command.Command):
         )
 
 
-class AddFloatingIP(command.Command):
+class AddFloatingIP(network_common.NetworkAndComputeCommand):
     _description = _("Add floating IP address to server")
 
-    def get_parser(self, prog_name):
-        parser = super(AddFloatingIP, self).get_parser(prog_name)
+    def update_parser_common(self, parser):
         parser.add_argument(
             "server",
             metavar="<server>",
@@ -252,19 +245,37 @@ class AddFloatingIP(command.Command):
         parser.add_argument(
             "--fixed-ip-address",
             metavar="<ip-address>",
-            help=_("Fixed IP address to associate with this floating IP "
-                   "address"),
+            help=_(
+                "Fixed IP address to associate with this floating IP address"
+            ),
         )
         return parser
 
-    def take_action(self, parsed_args):
+    def take_action_network(self, client, parsed_args):
         compute_client = self.app.client_manager.compute
 
+        attrs = {}
+        obj = client.find_ip(
+            parsed_args.ip_address,
+            ignore_missing=False,
+        )
         server = utils.find_resource(
-            compute_client.servers, parsed_args.server)
+            compute_client.servers,
+            parsed_args.server,
+        )
+        port = list(client.ports(device_id=server.id))[0]
+        attrs['port_id'] = port.id
+        if parsed_args.fixed_ip_address:
+            attrs['fixed_ip_address'] = parsed_args.fixed_ip_address
 
-        server.add_floating_ip(parsed_args.ip_address,
-                               parsed_args.fixed_ip_address)
+        client.update_ip(obj, **attrs)
+
+    def take_action_compute(self, client, parsed_args):
+        client.api.floating_ip_add(
+            parsed_args.server,
+            parsed_args.ip_address,
+            fixed_address=parsed_args.fixed_ip_address,
+        )
 
 
 class AddPort(command.Command):
@@ -300,6 +311,39 @@ class AddPort(command.Command):
         server.interface_attach(port_id=port_id, net_id=None, fixed_ip=None)
 
 
+class AddNetwork(command.Command):
+    _description = _("Add network to server")
+
+    def get_parser(self, prog_name):
+        parser = super(AddNetwork, self).get_parser(prog_name)
+        parser.add_argument(
+            "server",
+            metavar="<server>",
+            help=_("Server to add the network to (name or ID)"),
+        )
+        parser.add_argument(
+            "network",
+            metavar="<network>",
+            help=_("Network to add to the server (name or ID)"),
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        compute_client = self.app.client_manager.compute
+
+        server = utils.find_resource(
+            compute_client.servers, parsed_args.server)
+
+        if self.app.client_manager.is_network_endpoint_enabled():
+            network_client = self.app.client_manager.network
+            net_id = network_client.find_network(
+                parsed_args.network, ignore_missing=False).id
+        else:
+            net_id = parsed_args.network
+
+        server.interface_attach(port_id=None, net_id=net_id, fixed_ip=None)
+
+
 class AddServerSecurityGroup(command.Command):
     _description = _("Add security group to server")
 
@@ -547,6 +591,12 @@ class CreateServer(command.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         volume_client = self.app.client_manager.volume
         image_client = self.app.client_manager.image
@@ -781,11 +831,11 @@ class CreateServer(command.ShowOne):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write('\n')
+                self.app.stdout.write('\n')
             else:
                 LOG.error(_('Error creating server: %s'),
                           parsed_args.server_name)
-                sys.stdout.write(_('Error creating server\n'))
+                self.app.stdout.write(_('Error creating server\n'))
                 raise SystemExit
 
         details = _prep_server_detail(compute_client, image_client, server)
@@ -839,6 +889,12 @@ class DeleteServer(command.Command):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         for server in parsed_args.server:
             server_obj = utils.find_resource(
@@ -850,11 +906,11 @@ class DeleteServer(command.Command):
                     server_obj.id,
                     callback=_show_progress,
                 ):
-                    sys.stdout.write('\n')
+                    self.app.stdout.write('\n')
                 else:
                     LOG.error(_('Error deleting server: %s'),
                               server_obj.id)
-                    sys.stdout.write(_('Error deleting server\n'))
+                    self.app.stdout.write(_('Error deleting server\n'))
                     raise SystemExit
 
 
@@ -1023,17 +1079,18 @@ class ListServer(command.Lister):
             'all_tenants': parsed_args.all_projects,
             'user_id': user_id,
             'deleted': parsed_args.deleted,
-            'changes_since': parsed_args.changes_since,
+            'changes-since': parsed_args.changes_since,
         }
         LOG.debug('search options: %s', search_opts)
 
-        if search_opts['changes_since']:
+        if search_opts['changes-since']:
             try:
-                timeutils.parse_isotime(search_opts['changes_since'])
+                timeutils.parse_isotime(search_opts['changes-since'])
             except ValueError:
-                raise exceptions.CommandError(_('Invalid changes-since value:'
-                                              ' %s') % search_opts['changes'
-                                                                   '_since'])
+                raise exceptions.CommandError(
+                    _('Invalid changes-since value: %s') %
+                    search_opts['changes-since']
+                )
 
         if parsed_args.long:
             columns = (
@@ -1256,6 +1313,11 @@ class MigrateServer(command.Command):
 
     def take_action(self, parsed_args):
 
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
 
         server = utils.find_resource(
@@ -1281,11 +1343,11 @@ class MigrateServer(command.Command):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('Complete\n'))
+                self.app.stdout.write(_('Complete\n'))
             else:
                 LOG.error(_('Error migrating server: %s'),
                           server.id)
-                sys.stdout.write(_('Error migrating server\n'))
+                self.app.stdout.write(_('Error migrating server\n'))
                 raise SystemExit
 
 
@@ -1346,6 +1408,12 @@ class RebootServer(command.Command):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         server = utils.find_resource(
             compute_client.servers, parsed_args.server)
@@ -1357,11 +1425,11 @@ class RebootServer(command.Command):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('Complete\n'))
+                self.app.stdout.write(_('Complete\n'))
             else:
                 LOG.error(_('Error rebooting server: %s'),
                           server.id)
-                sys.stdout.write(_('Error rebooting server\n'))
+                self.app.stdout.write(_('Error rebooting server\n'))
                 raise SystemExit
 
 
@@ -1394,6 +1462,12 @@ class RebuildServer(command.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         image_client = self.app.client_manager.image
 
@@ -1411,11 +1485,11 @@ class RebuildServer(command.ShowOne):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('Complete\n'))
+                self.app.stdout.write(_('Complete\n'))
             else:
                 LOG.error(_('Error rebuilding server: %s'),
                           server.id)
-                sys.stdout.write(_('Error rebuilding server\n'))
+                self.app.stdout.write(_('Error rebuilding server\n'))
                 raise SystemExit
 
         details = _prep_server_detail(compute_client, image_client, server)
@@ -1448,11 +1522,10 @@ class RemoveFixedIP(command.Command):
         server.remove_fixed_ip(parsed_args.ip_address)
 
 
-class RemoveFloatingIP(command.Command):
+class RemoveFloatingIP(network_common.NetworkAndComputeCommand):
     _description = _("Remove floating IP address from server")
 
-    def get_parser(self, prog_name):
-        parser = super(RemoveFloatingIP, self).get_parser(prog_name)
+    def update_parser_common(self, parser):
         parser.add_argument(
             "server",
             metavar="<server>",
@@ -1467,13 +1540,21 @@ class RemoveFloatingIP(command.Command):
         )
         return parser
 
-    def take_action(self, parsed_args):
-        compute_client = self.app.client_manager.compute
+    def take_action_network(self, client, parsed_args):
+        attrs = {}
+        obj = client.find_ip(
+            parsed_args.ip_address,
+            ignore_missing=False,
+        )
+        attrs['port_id'] = None
 
-        server = utils.find_resource(
-            compute_client.servers, parsed_args.server)
+        client.update_ip(obj, **attrs)
 
-        server.remove_floating_ip(parsed_args.ip_address)
+    def take_action_compute(self, client, parsed_args):
+        client.api.floating_ip_remove(
+            parsed_args.server,
+            parsed_args.ip_address,
+        )
 
 
 class RemovePort(command.Command):
@@ -1509,6 +1590,41 @@ class RemovePort(command.Command):
         server.interface_detach(port_id)
 
 
+class RemoveNetwork(command.Command):
+    _description = _("Remove all ports of a network from server")
+
+    def get_parser(self, prog_name):
+        parser = super(RemoveNetwork, self).get_parser(prog_name)
+        parser.add_argument(
+            "server",
+            metavar="<server>",
+            help=_("Server to remove the port from (name or ID)"),
+        )
+        parser.add_argument(
+            "network",
+            metavar="<network>",
+            help=_("Network to remove from the server (name or ID)"),
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        compute_client = self.app.client_manager.compute
+
+        server = utils.find_resource(
+            compute_client.servers, parsed_args.server)
+
+        if self.app.client_manager.is_network_endpoint_enabled():
+            network_client = self.app.client_manager.network
+            net_id = network_client.find_network(
+                parsed_args.network, ignore_missing=False).id
+        else:
+            net_id = parsed_args.network
+
+        for inf in server.interface_list():
+            if inf.net_id == net_id:
+                server.interface_detach(inf.port_id)
+
+
 class RemoveServerSecurityGroup(command.Command):
     _description = _("Remove security group from server")
 
@@ -1576,7 +1692,7 @@ class RemoveServerVolume(command.Command):
         )
 
 
-class RescueServer(command.ShowOne):
+class RescueServer(command.Command):
     _description = _("Put server in rescue mode")
 
     def get_parser(self, prog_name):
@@ -1586,16 +1702,35 @@ class RescueServer(command.ShowOne):
             metavar='<server>',
             help=_('Server (name or ID)'),
         )
+        parser.add_argument(
+            '--image',
+            metavar='<image>',
+            help=_('Image (name or ID) to use for the rescue mode.'
+                   ' Defaults to the currently used one.'),
+        )
+        parser.add_argument(
+            '--password',
+            metavar='<password>',
+            help=_("Set the password on the rescued instance"),
+        )
         return parser
 
     def take_action(self, parsed_args):
-
         compute_client = self.app.client_manager.compute
-        _, body = utils.find_resource(
+        image_client = self.app.client_manager.image
+
+        image = None
+        if parsed_args.image:
+            image = utils.find_resource(
+                image_client.images,
+                parsed_args.image,
+            )
+
+        utils.find_resource(
             compute_client.servers,
             parsed_args.server,
-        ).rescue()
-        return zip(*sorted(six.iteritems(body)))
+        ).rescue(image=image,
+                 password=parsed_args.password)
 
 
 class ResizeServer(command.Command):
@@ -1639,6 +1774,11 @@ the new server and restart the old one.""")
 
     def take_action(self, parsed_args):
 
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         server = utils.find_resource(
             compute_client.servers,
@@ -1657,11 +1797,11 @@ the new server and restart the old one.""")
                     success_status=['active', 'verify_resize'],
                     callback=_show_progress,
                 ):
-                    sys.stdout.write(_('Complete\n'))
+                    self.app.stdout.write(_('Complete\n'))
                 else:
                     LOG.error(_('Error resizing server: %s'),
                               server.id)
-                    sys.stdout.write(_('Error resizing server\n'))
+                    self.app.stdout.write(_('Error resizing server\n'))
                     raise SystemExit
         elif parsed_args.confirm:
             compute_client.servers.confirm_resize(server)
@@ -1827,7 +1967,9 @@ class ShowServer(command.ShowOne):
         if parsed_args.diagnostics:
             (resp, data) = server.diagnostics()
             if not resp.status_code == 200:
-                sys.stderr.write(_("Error retrieving diagnostics data\n"))
+                self.app.stderr.write(_(
+                    "Error retrieving diagnostics data\n"
+                ))
                 return ({}, {})
         else:
             data = _prep_server_detail(compute_client,
