@@ -205,6 +205,16 @@ class CreateImage(command.ShowOne):
             action="store_true",
             help=_("Image is inaccessible to the public (default)"),
         )
+        public_group.add_argument(
+            "--community",
+            action="store_true",
+            help=_("Image is accessible to the community"),
+        )
+        public_group.add_argument(
+            "--shared",
+            action="store_true",
+            help=_("Image can be shared"),
+        )
         parser.add_argument(
             "--property",
             dest="properties",
@@ -260,7 +270,7 @@ class CreateImage(command.ShowOne):
         kwargs = {}
         copy_attrs = ('name', 'id',
                       'container_format', 'disk_format',
-                      'min_disk', 'min_ram', 'tags')
+                      'min_disk', 'min_ram', 'tags', 'visibility')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
@@ -288,7 +298,10 @@ class CreateImage(command.ShowOne):
             kwargs['visibility'] = 'public'
         if parsed_args.private:
             kwargs['visibility'] = 'private'
-
+        if parsed_args.community:
+            kwargs['visibility'] = 'community'
+        if parsed_args.shared:
+            kwargs['visibility'] = 'shared'
         # Handle deprecated --owner option
         project_arg = parsed_args.project
         if parsed_args.owner:
@@ -439,6 +452,18 @@ class ListImage(command.Lister):
             action=parseractions.KeyValueAction,
             help=_('Filter output based on property'),
         )
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            default=None,
+            help=_("Filter images based on name.")
+        )
+        parser.add_argument(
+            '--status',
+            metavar='<status>',
+            default=None,
+            help=_("Filter images based on status.")
+        )
         parser.add_argument(
             '--long',
             action='store_true',
@@ -463,17 +488,17 @@ class ListImage(command.Lister):
         )
         parser.add_argument(
             "--limit",
-            metavar="<limit>",
+            metavar="<num-images>",
             type=int,
             help=_("Maximum number of images to display."),
         )
         parser.add_argument(
             '--marker',
-            metavar='<marker>',
+            metavar='<image>',
             default=None,
-            help=_("The last image (name or ID) of the previous page. Display "
+            help=_("The last image of the previous page. Display "
                    "list of images after marker. Display all images if not "
-                   "specified."),
+                   "specified. (name or ID)"),
         )
         return parser
 
@@ -492,6 +517,10 @@ class ListImage(command.Lister):
         if parsed_args.marker:
             kwargs['marker'] = utils.find_resource(image_client.images,
                                                    parsed_args.marker).id
+        if parsed_args.name:
+            kwargs['name'] = parsed_args.name
+        if parsed_args.status:
+            kwargs['status'] = parsed_args.status
         if parsed_args.long:
             columns = (
                 'ID',
@@ -620,6 +649,12 @@ class SaveImage(command.Command):
         )
         data = image_client.images.data(image.id)
 
+        if data.wrapped is None:
+            msg = _('Image %s has no data.') % image.id
+            LOG.error(msg)
+            self.app.stdout.write(msg + '\n')
+            raise SystemExit
+
         gc_utils.save_image(data, parsed_args.file)
 
 
@@ -698,6 +733,16 @@ class SetImage(command.Command):
             action="store_true",
             help=_("Image is inaccessible to the public (default)"),
         )
+        public_group.add_argument(
+            "--community",
+            action="store_true",
+            help=_("Image is accessible to the community"),
+        )
+        public_group.add_argument(
+            "--shared",
+            action="store_true",
+            help=_("Image can be shared"),
+        )
         parser.add_argument(
             "--property",
             dest="properties",
@@ -710,7 +755,7 @@ class SetImage(command.Command):
             "--tag",
             dest="tags",
             metavar="<tag>",
-            default=[],
+            default=None,
             action='append',
             help=_("Set a tag on this image "
                    "(repeat option to set multiple tags)"),
@@ -817,11 +862,11 @@ class SetImage(command.Command):
         copy_attrs = ('architecture', 'container_format', 'disk_format',
                       'file', 'instance_id', 'kernel_id', 'locations',
                       'min_disk', 'min_ram', 'name', 'os_distro', 'os_version',
-                      'prefix', 'progress', 'ramdisk_id', 'tags')
+                      'prefix', 'progress', 'ramdisk_id', 'tags', 'visibility')
         for attr in copy_attrs:
             if attr in parsed_args:
                 val = getattr(parsed_args, attr, None)
-                if val:
+                if val is not None:
                     # Only include a value in kwargs for attributes that are
                     # actually present on the command line
                     kwargs[attr] = val
@@ -845,7 +890,10 @@ class SetImage(command.Command):
             kwargs['visibility'] = 'public'
         if parsed_args.private:
             kwargs['visibility'] = 'private'
-
+        if parsed_args.community:
+            kwargs['visibility'] = 'community'
+        if parsed_args.shared:
+            kwargs['visibility'] = 'shared'
         # Handle deprecated --owner option
         project_arg = parsed_args.project
         if parsed_args.owner:
@@ -905,6 +953,12 @@ class ShowImage(command.ShowOne):
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
+        parser.add_argument(
+            "--human-readable",
+            default=False,
+            action='store_true',
+            help=_("Print image size in a human-friendly format."),
+        )
         parser.add_argument(
             "image",
             metavar="<image>",
@@ -918,6 +972,8 @@ class ShowImage(command.ShowOne):
             image_client.images,
             parsed_args.image,
         )
+        if parsed_args.human_readable:
+            image['size'] = utils.format_size(image['size'])
 
         info = _format_image(image)
         return zip(*sorted(six.iteritems(info)))
@@ -940,16 +996,16 @@ class UnsetImage(command.Command):
             default=[],
             action='append',
             help=_("Unset a tag on this image "
-                   "(repeat option to set multiple tags)"),
+                   "(repeat option to unset multiple tags)"),
         )
         parser.add_argument(
             "--property",
             dest="properties",
-            metavar="<property_key>",
+            metavar="<property-key>",
             default=[],
             action='append',
             help=_("Unset a property on this image "
-                   "(repeat option to set multiple properties)"),
+                   "(repeat option to unset multiple properties)"),
         )
         return parser
 
@@ -974,9 +1030,7 @@ class UnsetImage(command.Command):
 
         if parsed_args.properties:
             for k in parsed_args.properties:
-                try:
-                    assert(k in image.keys())
-                except AssertionError:
+                if k not in image:
                     LOG.error(_("property unset failed, '%s' is a "
                                 "nonexistent property "), k)
                     propret += 1
