@@ -127,6 +127,8 @@ def _get_attrs(client_manager, parsed_args):
     if parsed_args.mac_address is not None:
         attrs['mac_address'] = parsed_args.mac_address
 
+    if parsed_args.dns_domain is not None:
+        attrs['dns_domain'] = parsed_args.dns_domain
     if parsed_args.dns_name is not None:
         attrs['dns_name'] = parsed_args.dns_name
     # It is possible that name is not updated during 'port set'
@@ -268,6 +270,12 @@ def _add_updatable_args(parser):
         metavar='<host-id>',
         help=argparse.SUPPRESS,
     )
+    parser.add_argument(
+        '--dns-domain',
+        metavar='dns-domain',
+        help=_("Set DNS domain to this port "
+               "(requires dns_domain extension for ports)")
+    )
     parser.add_argument(
         '--dns-name',
         metavar='dns-name',
@@ -499,6 +507,11 @@ class ListPort(command.Lister):
             metavar='<server>',
             help=_("List only ports attached to this server (name or ID)"),
         )
+        device_group.add_argument(
+            '--device-id',
+            metavar='<device-id>',
+            help=_("List only ports with the specified device ID")
+        )
         parser.add_argument(
             '--mac-address',
             metavar='<mac-address>',
@@ -553,6 +566,8 @@ class ListPort(command.Lister):
             column_headers += ('Security Groups', 'Device Owner', 'Tags')
         if parsed_args.device_owner is not None:
             filters['device_owner'] = parsed_args.device_owner
+        if parsed_args.device_id is not None:
+            filters['device_id'] = parsed_args.device_id
         if parsed_args.router:
             _router = network_client.find_router(parsed_args.router,
                                                  ignore_missing=False)
@@ -584,9 +599,11 @@ class ListPort(command.Lister):
 
         data = network_client.ports(**filters)
 
-        return (column_headers,
+        headers, attrs = utils.calculate_header_and_attrs(
+            column_headers, columns, parsed_args)
+        return (headers,
                 (utils.get_item_properties(
-                    s, columns,
+                    s, attrs,
                     formatters=_formatters,
                 ) for s in data))
 
