@@ -14,9 +14,10 @@
 #
 
 import copy
-import mock
 import uuid
 
+import mock
+
 from openstackclient.api import compute_v2
 from openstackclient.tests.unit import fakes
 from openstackclient.tests.unit.identity.v2_0 import fakes as identity_fakes
@@ -1229,10 +1230,7 @@ class FakeHost(object):
             'project': 'project-' + uuid.uuid4().hex,
         }
         host_info.update(attrs)
-        host = fakes.FakeResource(
-            info=copy.deepcopy(host_info),
-            loaded=True)
-        return host
+        return host_info
 
 
 class FakeServerGroup(object):
