@@ -160,7 +160,7 @@ class TestFlavorCreate(TestFlavor):
 
         self.flavor.is_public = False
         arglist = [
-            '--id', self.flavor.id,
+            '--id', 'auto',
             '--ram', str(self.flavor.ram),
             '--disk', str(self.flavor.disk),
             '--ephemeral', str(self.flavor.ephemeral),
@@ -174,7 +174,6 @@ class TestFlavorCreate(TestFlavor):
             self.flavor.name,
         ]
         verifylist = [
-            ('id', self.flavor.id),
             ('ram', self.flavor.ram),
             ('disk', self.flavor.disk),
             ('ephemeral', self.flavor.ephemeral),
@@ -193,7 +192,7 @@ class TestFlavorCreate(TestFlavor):
             self.flavor.ram,
             self.flavor.vcpus,
             self.flavor.disk,
-            self.flavor.id,
+            'auto',
             self.flavor.ephemeral,
             self.flavor.swap,
             self.flavor.rxtx_factor,
@@ -529,6 +528,23 @@ class TestFlavorSet(TestFlavor):
         self.flavor.set_keys.assert_called_with({'FOO': '"B A R"'})
         self.assertIsNone(result)
 
+    def test_flavor_set_no_property(self):
+        arglist = [
+            '--no-property',
+            'baremetal'
+        ]
+        verifylist = [
+            ('no_property', True),
+            ('flavor', 'baremetal')
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.flavors_mock.find.assert_called_with(name=parsed_args.flavor,
+                                                  is_public=None)
+        self.flavor.unset_keys.assert_called_with(['property'])
+        self.assertIsNone(result)
+
     def test_flavor_set_project(self):
         arglist = [
             '--project', self.project.id,
