@@ -12,10 +12,11 @@
 #
 
 import argparse
-import mock
 
+import mock
 import openstack
-from openstackclient.common import exceptions
+from osc_lib import exceptions
+
 from openstackclient.network import common
 from openstackclient.tests.unit import utils
 
