@@ -62,6 +62,7 @@ class TestCreateFloatingIPNetwork(TestFloatingIPNetwork):
         'id',
         'port_id',
         'project_id',
+        'qos_policy_id',
         'router_id',
         'status',
     )
@@ -76,6 +77,7 @@ class TestCreateFloatingIPNetwork(TestFloatingIPNetwork):
         floating_ip.id,
         floating_ip.port_id,
         floating_ip.project_id,
+        floating_ip.qos_policy_id,
         floating_ip.router_id,
         floating_ip.status,
     )
@@ -197,6 +199,28 @@ class TestCreateFloatingIPNetwork(TestFloatingIPNetwork):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
+    def test_create_floating_ip_with_qos(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        arglist = [
+            '--qos-policy', qos_policy.id,
+            self.floating_ip.floating_network_id,
+        ]
+        verifylist = [
+            ('network', self.floating_ip.floating_network_id),
+            ('qos_policy', qos_policy.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.create_ip.assert_called_once_with(**{
+            'floating_network_id': self.floating_ip.floating_network_id,
+            'qos_policy_id': qos_policy.id,
+        })
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
 
 class TestDeleteFloatingIPNetwork(TestFloatingIPNetwork):
 
@@ -206,14 +230,14 @@ class TestDeleteFloatingIPNetwork(TestFloatingIPNetwork):
     def setUp(self):
         super(TestDeleteFloatingIPNetwork, self).setUp()
 
+        self.network.find_ip = mock.Mock()
         self.network.delete_ip = mock.Mock(return_value=None)
 
         # Get the command object to test
         self.cmd = fip.DeleteFloatingIP(self.app, self.namespace)
 
-    @mock.patch.object(fip, '_find_floating_ip')
-    def test_floating_ip_delete(self, find_floating_ip_mock):
-        find_floating_ip_mock.side_effect = [
+    def test_floating_ip_delete(self):
+        self.network.find_ip.side_effect = [
             self.floating_ips[0],
             self.floating_ips[1],
         ]
@@ -227,17 +251,15 @@ class TestDeleteFloatingIPNetwork(TestFloatingIPNetwork):
 
         result = self.cmd.take_action(parsed_args)
 
-        find_floating_ip_mock.assert_called_once_with(
-            mock.ANY,
+        self.network.find_ip.assert_called_once_with(
             self.floating_ips[0].id,
             ignore_missing=False,
         )
         self.network.delete_ip.assert_called_once_with(self.floating_ips[0])
         self.assertIsNone(result)
 
-    @mock.patch.object(fip, '_find_floating_ip')
-    def test_floating_ip_delete_multi(self, find_floating_ip_mock):
-        find_floating_ip_mock.side_effect = [
+    def test_floating_ip_delete_multi(self):
+        self.network.find_ip.side_effect = [
             self.floating_ips[0],
             self.floating_ips[1],
         ]
@@ -255,17 +277,15 @@ class TestDeleteFloatingIPNetwork(TestFloatingIPNetwork):
 
         calls = [
             call(
-                mock.ANY,
                 self.floating_ips[0].id,
                 ignore_missing=False,
             ),
             call(
-                mock.ANY,
                 self.floating_ips[1].id,
                 ignore_missing=False,
             ),
         ]
-        find_floating_ip_mock.assert_has_calls(calls)
+        self.network.find_ip.assert_has_calls(calls)
 
         calls = []
         for f in self.floating_ips:
@@ -273,9 +293,8 @@ class TestDeleteFloatingIPNetwork(TestFloatingIPNetwork):
         self.network.delete_ip.assert_has_calls(calls)
         self.assertIsNone(result)
 
-    @mock.patch.object(fip, '_find_floating_ip')
-    def test_floating_ip_delete_multi_exception(self, find_floating_ip_mock):
-        find_floating_ip_mock.side_effect = [
+    def test_floating_ip_delete_multi_exception(self):
+        self.network.find_ip.side_effect = [
             self.floating_ips[0],
             exceptions.CommandError,
         ]
@@ -295,13 +314,11 @@ class TestDeleteFloatingIPNetwork(TestFloatingIPNetwork):
         except exceptions.CommandError as e:
             self.assertEqual('1 of 2 floating_ips failed to delete.', str(e))
 
-        find_floating_ip_mock.assert_any_call(
-            mock.ANY,
+        self.network.find_ip.assert_any_call(
             self.floating_ips[0].id,
             ignore_missing=False,
         )
-        find_floating_ip_mock.assert_any_call(
-            mock.ANY,
+        self.network.find_ip.assert_any_call(
             'unexist_floating_ip',
             ignore_missing=False,
         )
@@ -538,6 +555,7 @@ class TestShowFloatingIPNetwork(TestFloatingIPNetwork):
         'id',
         'port_id',
         'project_id',
+        'qos_policy_id',
         'router_id',
         'status',
     )
@@ -552,6 +570,7 @@ class TestShowFloatingIPNetwork(TestFloatingIPNetwork):
         floating_ip.id,
         floating_ip.port_id,
         floating_ip.project_id,
+        floating_ip.qos_policy_id,
         floating_ip.router_id,
         floating_ip.status,
     )
@@ -564,9 +583,7 @@ class TestShowFloatingIPNetwork(TestFloatingIPNetwork):
         # Get the command object to test
         self.cmd = fip.ShowFloatingIP(self.app, self.namespace)
 
-    @mock.patch.object(fip, '_find_floating_ip')
-    def test_floating_ip_show(self, find_floating_ip_mock):
-        find_floating_ip_mock.return_value = self.floating_ip
+    def test_floating_ip_show(self):
         arglist = [
             self.floating_ip.id,
         ]
@@ -577,8 +594,7 @@ class TestShowFloatingIPNetwork(TestFloatingIPNetwork):
 
         columns, data = self.cmd.take_action(parsed_args)
 
-        find_floating_ip_mock.assert_called_once_with(
-            mock.ANY,
+        self.network.find_ip.assert_called_once_with(
             self.floating_ip.id,
             ignore_missing=False,
         )
@@ -610,14 +626,7 @@ class TestSetFloatingIP(TestFloatingIPNetwork):
         # Get the command object to test
         self.cmd = fip.SetFloatingIP(self.app, self.namespace)
 
-    @mock.patch(
-        "openstackclient.tests.unit.network.v2.test_floating_ip_network." +
-        "fip._find_floating_ip"
-    )
-    def test_port_option(self, find_floating_ip_mock):
-        find_floating_ip_mock.side_effect = [
-            self.floating_ip,
-        ]
+    def test_port_option(self):
         arglist = [
             self.floating_ip.id,
             '--port', self.floating_ip.port_id,
@@ -634,8 +643,7 @@ class TestSetFloatingIP(TestFloatingIPNetwork):
             'port_id': self.floating_ip.port_id,
         }
 
-        find_floating_ip_mock.assert_called_once_with(
-            mock.ANY,
+        self.network.find_ip.assert_called_once_with(
             self.floating_ip.id,
             ignore_missing=False,
         )
@@ -643,14 +651,7 @@ class TestSetFloatingIP(TestFloatingIPNetwork):
         self.network.update_ip.assert_called_once_with(
             self.floating_ip, **attrs)
 
-    @mock.patch(
-        "openstackclient.tests.unit.network.v2.test_floating_ip_network." +
-        "fip._find_floating_ip"
-    )
-    def test_fixed_ip_option(self, find_floating_ip_mock):
-        find_floating_ip_mock.side_effect = [
-            self.floating_ip,
-        ]
+    def test_fixed_ip_option(self):
         arglist = [
             self.floating_ip.id,
             '--port', self.floating_ip.port_id,
@@ -669,8 +670,61 @@ class TestSetFloatingIP(TestFloatingIPNetwork):
             'port_id': self.floating_ip.port_id,
             'fixed_ip_address': self.floating_ip.fixed_ip_address,
         }
-        find_floating_ip_mock.assert_called_once_with(
-            mock.ANY,
+        self.network.find_ip.assert_called_once_with(
+            self.floating_ip.id,
+            ignore_missing=False,
+        )
+        self.network.update_ip.assert_called_once_with(
+            self.floating_ip, **attrs)
+
+    def test_port_and_qos_policy_option(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        arglist = [
+            "--qos-policy", qos_policy.id,
+            '--port', self.floating_ip.port_id,
+            self.floating_ip.id,
+        ]
+        verifylist = [
+            ('qos_policy', qos_policy.id),
+            ('port', self.floating_ip.port_id),
+            ('floating_ip', self.floating_ip.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'qos_policy_id': qos_policy.id,
+            'port_id': self.floating_ip.port_id,
+        }
+        self.network.find_ip.assert_called_once_with(
+            self.floating_ip.id,
+            ignore_missing=False,
+        )
+        self.network.update_ip.assert_called_once_with(
+            self.floating_ip, **attrs)
+
+    def test_port_and_no_qos_policy_option(self):
+        arglist = [
+            "--no-qos-policy",
+            '--port', self.floating_ip.port_id,
+            self.floating_ip.id,
+        ]
+        verifylist = [
+            ('no_qos_policy', True),
+            ('port', self.floating_ip.port_id),
+            ('floating_ip', self.floating_ip.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'qos_policy_id': None,
+            'port_id': self.floating_ip.port_id,
+        }
+        self.network.find_ip.assert_called_once_with(
             self.floating_ip.id,
             ignore_missing=False,
         )
@@ -700,14 +754,7 @@ class TestUnsetFloatingIP(TestFloatingIPNetwork):
         # Get the command object to test
         self.cmd = fip.UnsetFloatingIP(self.app, self.namespace)
 
-    @mock.patch(
-        "openstackclient.tests.unit.network.v2.test_floating_ip_network." +
-        "fip._find_floating_ip"
-    )
-    def test_floating_ip_unset_port(self, find_floating_ip_mock):
-        find_floating_ip_mock.side_effect = [
-            self.floating_ip,
-        ]
+    def test_floating_ip_unset_port(self):
         arglist = [
             self.floating_ip.id,
             "--port",
@@ -723,8 +770,32 @@ class TestUnsetFloatingIP(TestFloatingIPNetwork):
         attrs = {
             'port_id': None,
         }
-        find_floating_ip_mock.assert_called_once_with(
-            mock.ANY,
+        self.network.find_ip.assert_called_once_with(
+            self.floating_ip.id,
+            ignore_missing=False,
+        )
+        self.network.update_ip.assert_called_once_with(
+            self.floating_ip, **attrs)
+
+        self.assertIsNone(result)
+
+    def test_floating_ip_unset_qos_policy(self):
+        arglist = [
+            self.floating_ip.id,
+            "--qos-policy",
+        ]
+        verifylist = [
+            ('floating_ip', self.floating_ip.id),
+            ('qos_policy', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'qos_policy_id': None,
+        }
+        self.network.find_ip.assert_called_once_with(
             self.floating_ip.id,
             ignore_missing=False,
         )
