@@ -95,6 +95,9 @@ class CreateQos(command.ShowOne):
 
         qos_spec = volume_client.qos_specs.create(parsed_args.name, specs)
 
+        qos_spec._info.update(
+            {'properties': utils.format_dict(qos_spec._info.pop('specs'))}
+        )
         return zip(*sorted(six.iteritems(qos_spec._info)))
 
 
@@ -184,14 +187,26 @@ class ListQos(command.Lister):
         qos_specs_list = volume_client.qos_specs.list()
 
         for qos in qos_specs_list:
-            qos_associations = volume_client.qos_specs.get_associations(qos)
-            if qos_associations:
-                associations = [association.name
-                                for association in qos_associations]
-                qos._info.update({'associations': associations})
+            try:
+                qos_associations = volume_client.qos_specs.get_associations(
+                    qos,
+                )
+                if qos_associations:
+                    associations = [
+                        association.name for association in qos_associations
+                    ]
+                    qos._info.update({'associations': associations})
+            except Exception as ex:
+                if type(ex).__name__ == 'NotFound':
+                    qos._info.update({'associations': None})
+                else:
+                    raise
+
+        display_columns = (
+            'ID', 'Name', 'Consumer', 'Associations', 'Properties')
 
         columns = ('ID', 'Name', 'Consumer', 'Associations', 'Specs')
-        return (columns,
+        return (display_columns,
                 (utils.get_dict_properties(
                     s._info, columns,
                     formatters={
@@ -254,7 +269,8 @@ class ShowQos(command.ShowOne):
             qos_spec._info.update({
                 'associations': utils.format_list(associations)
             })
-        qos_spec._info.update({'specs': utils.format_dict(qos_spec.specs)})
+        qos_spec._info.update(
+            {'properties': utils.format_dict(qos_spec._info.pop('specs'))})
 
         return zip(*sorted(six.iteritems(qos_spec._info)))
 
